%% program computes the attitudes of an agent with 
%% first-order risk averse preferences (R-FORA)
%% to a delayed gamble
%% creates the 'x' signs in Figure 2

%% mean and standard deviation of the log return
%% on the non-financial asset
rm = 0.04;
rs = 0.03;
%% gross risk-free rate
rf = 1.02;
%% fraction of wealth theta_n_bar (th) invested
%% in the non-financial asset
th = 0.75;

%% parameters of the aggregator function W(.,.)
%% beta (bet) and rho (rho)
bet = 0.9;
rho = -1;

%% Figure 2 looks at a range of values of the
%% preference parameters gamma (gm) and lambda (lam)
gm_min = 1.1;
gm_max = 150;
lam_min = 1.1;
lam_max = 150;

%% fineness of grid discretizing the (gamma,lambda) plane
lim = 40;

%% fineness of grid used in Simpson's rule numerical integration
grd_fin = 1001;
%% the weighting vector for Simpson's rule
t1 = 4*kron(ones((grd_fin-3)/2,1),[1;0])+2*kron(ones((grd_fin-3)/2,1),[0;1]);
t1 = [1; t1; 4; 1];

%% array marking range of (gamma,lambda) plane for which
%% various gambles are accepted/rejected
gmb3 = zeros(lim,lim);

%% iterate through each (gamma,lambda) pair on the grid
for count = 1:lim;
   count
   for count2 = 1:lim;
      gm = gm_min + (count-1)*(gm_max-gm_min)/(lim-1);
      lam = lam_min + (count2-1)*(lam_max-lam_min)/(lim-1);

%% STEP ONE

%% first compute mu(R), in equation (40)
%% the certainty equivalent of the background risk

stp = 20/(grd_fin-1);
grd = [-10:stp:10]';

%% compute the certainty equivalent mu(R) in equation (40) 
%% certainty equivalent is defined in equation (4)
u1 = (1-th)*rf+th*exp(rm+rs*grd);
u2 = (u1.^(1-gm)).*exp(-grd.^2/2);
u3 = (u2'*t1)*stp/(3*sqrt(2*pi));
%% starting values for search
m_0 = 0.9;
m_1 = 1.15;
%% now compute the function in (4) at each endpoint
%% first endpoint
w1 = u1.^(1-gm) - m_0^(1-gm);
w2 = (1-sign(u1-m_0))/2;
w3 = w1.*w2.*exp(-grd.^2/2);
w4 = (w3'*t1)*stp/(3*sqrt(2*pi));
f_0 = u3 + (lam-1)*w4 - m_0^(1-gm);
%% second endpoint
w1 = u1.^(1-gm) - m_1^(1-gm);
w2 = (1-sign(u1-m_1))/2;
w3 = w1.*w2.*exp(-grd.^2/2);
w4 = (w3'*t1)*stp/(3*sqrt(2*pi));
f_1 = u3 + (lam-1)*w4 - m_1^(1-gm);
%% now use interval bisection to find the certainty equivalent
flg = 1;
ct = 1;
while flg > 0.0000001;
    ct = ct + 1;
    m_2 = (m_0+m_1)/2;
    flg = abs(m_2-m_1);
    w1 = u1.^(1-gm)-m_2^(1-gm);
    w2 = (1-sign(u1-m_2))/2;
    w3 = w1.*w2.*exp(-grd.^2/2);
    w4 = (w3'*t1)*stp/(3*sqrt(2*pi));
    f_2 = u3 + (lam-1)*w4 - m_2^(1-gm);
    if sign(f_1) == sign(f_2);
        m_1 = m_2;
        f_1 = f_2;
    else;
        m_0 = m_1;
        m_1 = m_2;
        f_0 = f_1;
        f_1 = f_2;
    end;
end;

%% compute the consumption-wealth ratio alpha (aph) from equation (43)
aph = 1 - bet^(1/(1-rho))*m_1^(rho/(1-rho));
if aph < 0;
    error('need to lower beta for optimal consumption-wealth ratio to be positive');
end;
%% compute the value function constant A in equation (42)
A = (1-bet)^(1/rho)*aph^((rho-1)/rho);

%% STEP TWO

%% now solve problem (44) to see if G_L should be accepted
%% use an approximate technique to find optimum
%% approximation error is negligible

%% candidate consumption-wealth ratio after accepting G_L
aph2 = aph;

%% first, given the candidate consumption-wealth ration
%% compute the certainty equivalent of the risk when G_L is accepted
%% i.e. the mu term in equation (44)

%% this is the scaled version of G_L in equation (44)
x2 = 20000000/((1-aph2)*100000);
y2 = 10000/((1-aph2)*100000);

%% to compute the certainty equivalent, we need to integrate
%% numerically; use Simpson's rule

stp = 20/(grd_fin-1);
grd = [-10:stp:10]';

w1 = (((1-th)*rf+th*exp(rm+rs*grd)+x2).^(1-gm)).*exp(-grd.^2/2);
w1 = (w1'*t1)*stp/(3*sqrt(2*pi));

w2 = (max((1-th)*rf+th*exp(rm+rs*grd)-y2,0.0000001).^(1-gm)).*exp(-grd.^2/2);
w2 = (w2'*t1)*stp/(3*sqrt(2*pi));

%% lower and upper bounds for the certainty equivalent
mud_0 = exp(rm-10*rs);
mud_1 = exp(rm+10*rs);

%% we evaluate the certainty equivalent equation at these two bounds

for ct2 = 1:2;
   if ct2 == 1;
      mu = mud_0;
   else;
      mu = mud_1;
   end;
   %% know that the function being integrated will be zero beyond some point
   if mu > x2 + (1-th)*rf;
      nd1 = (log(mu-x2-(1-th)*rf)-log(th)-rm)/rs;
      n1 = min(10,nd1);
      stp3 = (n1+10)/(grd_fin-1);
      grd3 = [-10:stp3:n1]';
      w3 = (((1-th)*rf+th*exp(rm+rs*grd3)+x2).^(1-gm)).*exp(-grd3.^2/2);
      w3 = (w3'*t1)*stp3/(3*sqrt(2*pi));
   else;
      w3 = 0;
   end;
   if mu > -y2 + (1-th)*rf;
       nd2 = (log(mu+y2-(1-th)*rf)-log(th)-rm)/rs;
       n2 = min(10,nd2);
       stp4 = (n2+10)/(grd_fin-1);
       grd4 = [-10:stp4:n2]';
       w4 = (max((1-th)*rf+th*exp(rm+rs*grd4)-y2,0.0000001).^(1-gm)).*exp(-grd4.^2/2);
       w4 = (w4'*t1)*stp4/(3*sqrt(2*pi));
   else;
       w4 = 0;
   end;
   if mu > x2 + (1-th)*rf;
      w5 = erf(nd1/sqrt(2))/2+0.5;
   else;
      w5 = 0;
   end;
   if mu > -y2 + (1-th)*rf;
       w6 = erf(nd2/sqrt(2))/2+0.5;
   else;
       w6 = 0;
   end;
   if ct2 == 1;
      f_0 = 0.5*(w1+w2)+(lam-1)*0.5*(w3+w4-mu^(1-gm)*(w5+w6))-mu^(1-gm);
   else;
      f_1 = 0.5*(w1+w2)+(lam-1)*0.5*(w3+w4-mu^(1-gm)*(w5+w6))-mu^(1-gm);
   end;
end;

%% use interval bisection and Newton-Raphson to solve for certainty
%% equivalent

flg = 1;
ct2 = 1;

while flg > 0.00000000001;
   ct2 = ct2 + 1;
   if ct2 < 11;
      mud_2 = (mud_0+mud_1)/2;
   else;
      mud_2 = mud_0 - f_0*(mud_1-mud_0)/(f_1-f_0);
   end;
   flg = abs(mud_2-mud_1);
   if mud_2 > x2 + (1-th)*rf;
      nd1 = (log(mud_2-x2-(1-th)*rf)-log(th)-rm)/rs;
      n1 = min(10,nd1);
      stp3 = (n1+10)/(grd_fin-1);
      grd3 = [-10:stp3:n1]';
      w3 = (((1-th)*rf+th*exp(rm+rs*grd3)+x2).^(1-gm)).*exp(-grd3.^2/2);
      w3 = (w3'*t1)*stp3/(3*sqrt(2*pi));
   else;
      w3 = 0;
   end;
   if mud_2 > -y2 + (1-th)*rf;
       nd2 = (log(mud_2+y2-(1-th)*rf)-log(th)-rm)/rs;
       n2 = min(10,nd2);
       stp4 = (n2+10)/(grd_fin-1);
       grd4 = [-10:stp4:n2]';
       w4 = (max((1-th)*rf+th*exp(rm+rs*grd4)-y2,0.0000001).^(1-gm)).*exp(-grd4.^2/2);
       w4 = (w4'*t1)*stp4/(3*sqrt(2*pi));
   else;
       w4 = 0;
   end;
   if mud_2 > x2 + (1-th)*rf;
      w5 = erf(nd1/sqrt(2))/2+0.5;
   else;
      w5 = 0;
   end;
   if mud_2 > -y2 + (1-th)*rf;
       w6 = erf(nd2/sqrt(2))/2+0.5;
   else;
       w6 = 0;
   end;
   f_2 = 0.5*(w1+w2)+(lam-1)*0.5*(w3+w4-mud_2^(1-gm)*(w5+w6))-mud_2^(1-gm);   
   if sign(f_1) == sign(f_2);
      mud_1 = mud_2;
      f_1 = f_2;
   else;
      mud_0 = mud_1;
      mud_1 = mud_2;
      f_0 = f_1;
      f_1 = f_2;
   end;   
end;

mud2_s = A * mud_1;

%% generate the approximate optimal consumption-wealth ratio
aph2 = 1+((1-bet)/bet)^(1/(rho-1))*mud2_s^(-rho/(rho-1));
aph2 = 1/aph2;
%% generate the value function constant A_hat (A2)
A2 = (1-bet)^(1/rho)*aph2^((rho-1)/rho);

%% record range in which G_L is accepted
if A2 > A;
   gmb3(count,count2) = 1;
end;

end;
end;

%% plot of graph of the acceptance range

plt = [];
plt2 = [];

for count = 1:lim;
   for count2 = 1:lim;
      gm = gm_min+(count-1)*(gm_max-gm_min)/(lim-1);
      lam = lam_min+(count2-1)*(lam_max-lam_min)/(lim-1);
      if gmb3(count,count2) == 1;
         plt2 = [plt2; gm lam];
      end;
   end;
end;

figure;

hold on;
plot(plt2(:,1),plt2(:,2),'x');
hold off;

axis([1 gm_max 1 lam_max]);
xlabel('\gamma');
ylabel('\lambda');
